@@ -1,6 +1,6 @@
 <?php
 /**
- * $Horde: kronolith/attendees.php,v 1.7.8.8 2006/02/03 11:20:13 jan Exp $
+ * $Horde: kronolith/attendees.php,v 1.7.8.11 2006/07/19 17:12:12 jan Exp $
  *
  * Copyright 2004-2006 Code Fusion  <http://www.codefusion.co.za/>
  *                Stuart Binge <s.binge@codefusion.co.za>
@@ -41,6 +41,12 @@ case 'add':
     }
 
     $_SESSION['attendees'] = $attendees;
+
+    if (Util::getFormData('addNewClose')) {
+        Util::closeWindowJS();
+        exit;
+    }
+
     break;
 
 case 'remove':
@@ -109,31 +115,43 @@ $tabs->addTab(_("Week"), 'javascript:switchView(\'week\')', 'week');
 
 $attendee_view = &Kronolith_FreeBusy_View::singleton($view);
 
-// Add the creator as a required attendee in the Free/Busy display (if
-// the user has an email address configured).
-$myemail = $ident->getValue('from_addr');
-if (!empty($myemail)) {
-    $vfb = Kronolith::getFreeBusy($myemail);
-    if (!is_a($vfb, 'PEAR_Error')) {
-        $attendee_view->addRequiredMember($vfb);
-    } else {
-        $notification->push(sprintf(_("Error retrieving free/busy information for %s: %s"), $myemail, $vfb->getMessage()));
+// Add the creator as a required attendee in the Free/Busy display
+$cal = @unserialize($prefs->getValue('fb_cals'));
+if (!is_array($cal)) {
+    $cal = null;
+}
+
+// If the free/busy calendars preference is empty, default to the user's
+// default_share preference, and if that's empty, to their username.
+if (!$cal) {
+    $cal = $prefs->getValue('default_share');
+    if (!$cal) {
+        $cal = Auth::getAuth();
     }
+    $cal = array($cal);
+}
+$vfb = Kronolith::generateFreeBusy($cal, null, null, true);
+if (!is_a($vfb, 'PEAR_Error')) {
+    $attendee_view->addRequiredMember($vfb);
+} else {
+    $notification->push(sprintf(_("Error retrieving your free/busy information: %s"), $vfb->getMessage()));
 }
 
 // Add the Free/Busy information for each attendee.
 foreach ($attendees as $email => $status) {
-    if ($status['attendance'] == KRONOLITH_PART_REQUIRED) {
+    if ($status['attendance'] == KRONOLITH_PART_REQUIRED ||
+        $status['attendance'] == KRONOLITH_PART_OPTIONAL) {
         $vfb = Kronolith::getFreeBusy($email);
         if (!is_a($vfb, 'PEAR_Error')) {
-            $attendee_view->addRequiredMember($vfb);
-        } else {
-            $notification->push(sprintf(_("Error retrieving free/busy information for %s: %s"), $email, $vfb->getMessage()));
-        }
-    } elseif ($status['attendance'] == KRONOLITH_PART_OPTIONAL) {
-        $vfb = Kronolith::getFreeBusy($email);
-        if (!is_a($vfb, 'PEAR_Error')) {
-            $attendee_view->addOptionalMember($vfb);
+            $organizer = $vfb->getAttribute('ORGANIZER');
+            if (empty($organizer)) {
+                $vfb->setAttribute('ORGANIZER', 'MAILTO:' . $email, array(), false);
+            }
+            if ($status['attendance'] == KRONOLITH_PART_REQUIRED) {
+                $attendee_view->addRequiredMember($vfb);
+            } else {
+                $attendee_view->addOptionalMember($vfb);
+            }
         } else {
             $notification->push(sprintf(_("Error retrieving free/busy information for %s: %s"), $email, $vfb->getMessage()));
         }
