@@ -3,7 +3,7 @@
  * Kronolith_Driver:: defines an API for implementing storage backends
  * for Kronolith.
  *
- * $Horde: kronolith/lib/Driver.php,v 1.116.2.38 2006/03/01 19:31:42 chuck Exp $
+ * $Horde: kronolith/lib/Driver.php,v 1.116.2.55 2006/11/11 09:42:16 karsten Exp $
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @since   Kronolith 0.1
@@ -240,11 +240,11 @@ class Kronolith_Event {
     var $title = '';
 
     /**
-     * The identifier of the category this event belongs to.
+     * The category of this event.
      *
-     * @var integer
+     * @var string
      */
-    var $category = 0;
+    var $category = '';
 
     /**
      * The location this event occurs at.
@@ -402,11 +402,10 @@ class Kronolith_Event {
      *
      * @param Horde_iCalendar &$calendar  A Horde_iCalendar object that acts as
      *                                    a container.
-     * @param Identity $identity          The Identity object of the organizer.
      *
      * @return Horde_iCalendar_vevent  The vEvent object for this event.
      */
-    function &toiCalendar(&$calendar, $identity)
+    function &toiCalendar(&$calendar)
     {
         global $prefs;
 
@@ -417,7 +416,7 @@ class Kronolith_Event {
 
         if ($this->isAllDay()) {
             $vEvent->setAttribute('DTSTART', $this->start, array('VALUE' => 'DATE'));
-            $vEvent->setAttribute('DTEND', $this->end, array('VALUE' => 'DATE'));
+            $vEvent->setAttribute('DTEND', new Horde_Date($this->end->timestamp()-1), array('VALUE' => 'DATE'));
         } else {
             $vEvent->setAttribute('DTSTART', $this->start);
             $vEvent->setAttribute('DTEND', $this->end);
@@ -427,10 +426,9 @@ class Kronolith_Event {
         $vEvent->setAttribute('UID', $this->_uid);
         $vEvent->setAttribute('SUMMARY', $this->title);
         $vEvent->setAttribute('TRANSP', 'OPAQUE');
-        $name = $identity->getValue('fullname');
         $vEvent->setAttribute('ORGANIZER',
-                              'MAILTO:' . $identity->getValue('from_addr'),
-                              empty($name) ? null : array('CN' => $name));
+                              'MAILTO:' . Kronolith::getUserEmail($this->getCreatorId()),
+                              array('CN' => Kronolith::getUserName($this->getCreatorId())));
         if (!empty($this->description)) {
             $vEvent->setAttribute('DESCRIPTION', $this->description);
         }
@@ -481,7 +479,8 @@ class Kronolith_Event {
             $vEvent->setAttribute('ATTENDEE', 'MAILTO:' . $email, $params);
         }
 
-        // Alarm.
+        // vCalendar 1.0 alarms. Has to be replaced with vAlarm components for
+        // RFC 2445, if anyone ever requests.
         if (!empty($this->alarm)) {
             $vEvent->setAttribute('AALARM', $this->start->timestamp() - $this->alarm * 60);
         }
@@ -598,35 +597,130 @@ class Kronolith_Event {
 
         // Start and end date.
         $start = $vEvent->getAttribute('DTSTART');
-        if (!is_array($start) && !is_a($start, 'PEAR_Error')) {
-            $this->start = &new Horde_Date($start);
-        } elseif (is_array($start) && !is_a($start, 'PEAR_Error')) {
-            $this->start = &new Horde_Date(array('year' => (int)$start['year'],
-                                                 'month' => (int)$start['month'],
-                                                 'mday' => (int)$start['mday']));
+        if (!is_a($start, 'PEAR_Error')) {
+            if (!is_array($start)) {
+                // Date-Time field
+                $this->start = new Horde_Date($start);
+            } else {
+                // Date field
+                $this->start = new Horde_Date(array('year' => (int)$start['year'],
+                                                    'month' => (int)$start['month'],
+                                                    'mday' => (int)$start['mday']));
+            }
         }
         $end = $vEvent->getAttribute('DTEND');
-        if (!is_array($end) && !is_a($end, 'PEAR_Error')) {
-            $this->end = &new Horde_Date($end);
-        } elseif (is_array($end) && !is_a($end, 'PEAR_Error')) {
-            $this->end = &new Horde_Date(array('year' => (int)$end['year'],
-                                               'month' => (int)$end['month'],
-                                               'mday' => (int)$end['mday']));
+        if (!is_a($end, 'PEAR_Error')) {
+            if (!is_array($end)) {
+                // Date-Time field
+                $this->end = new Horde_Date($end);
+                /* All day events are transferred by many device as
+                 * DSTART: YYYYMMDDT000000 DTEND: YYYYMMDDT2359(59|00)
+                 * Convert accordingly */
+                if (is_object($this->start)
+                    && $this->start->hour == 0  && $this->start->min == 0
+                    && $this->start->sec == 0
+                    && $this->end->hour == 23 && $this->end->sec == 59) {
+                    $this->end = new Horde_Date(array(
+                                   'year' => (int)$this->end->year,
+                                   'month' => (int)$this->end->month,
+                                   'mday' => (int)$this->end->mday + 1));
+                    $this->end->correct();
+                }
+            } elseif (is_array($end) && !is_a($end, 'PEAR_Error')) {
+                // Date field
+                $this->end = new Horde_Date(array('year' => (int)$end['year'],
+                                                  'month' => (int)$end['month'],
+                                                  'mday' => (int)$end['mday'] + 1));
+                $this->end->correct();
+            }
         } else {
             $duration = $vEvent->getAttribute('DURATION');
             if (!is_array($duration) && !is_a($duration, 'PEAR_Error')) {
-                $this->end = &new Horde_Date($this->start->timestamp() + $duration);
+                $this->end = new Horde_Date($this->start->timestamp() + $duration);
             } else {
                 // End date equal to start date as per RFC 2445.
                 $this->end = Util::cloneObject($this->start);
             }
         }
 
+        /* vCalendar 1.0 alarms. */
         $alarm = $vEvent->getAttribute('AALARM');
         if (!is_array($alarm) && !is_a($alarm, 'PEAR_Error') && intval($alarm)) {
             $this->alarm = intval(($this->start->timestamp() - $alarm) / 60);
         }
 
+        /* Attendance.
+         * Importing attendance may result in confusion: editing an imported
+         * copy of an event can cause invitation updates to be sent from
+         * people other than the original organizer. So we don't import by
+         * default. However to allow updates by SyncML replication, the custom
+         * X-ATTENDEE attribute is used which has the same syntax as ATTENDEE.
+         */
+        $attendee = $vEvent->getAttribute('X-ATTENDEE');
+        if (!is_a($attendee, 'PEAR_Error')) {
+            require_once 'Horde/MIME.php';
+
+            if (!is_array($attendee)) {
+                $attendee = array($attendee);
+            }
+            $params = $vEvent->getAttribute('X-ATTENDEE', true);
+            if (!is_array($params)) {
+                $params = array($params);
+            }
+            for ($i = 0; $i < count($attendee); ++$i) {
+                $attendee[$i] = str_replace('MAILTO:', '', $attendee[$i]);
+                $email = MIME::bareAddress($attendee[$i]);
+                // Default according to rfc2445:
+                $attendance = KRONOLITH_PART_REQUIRED;
+                // vcalendar 2.0 style:
+                if (!empty($params[$i]['ROLE'])) {
+                    switch($params[$i]['ROLE']) {
+                    case 'OPT-PARTICIPANT':
+                        $attendance = KRONOLITH_PART_OPTIONAL;
+                        break;
+
+                    case 'NON-PARTICIPANT':
+                        $attendance = KRONOLITH_PART_NONE;
+                        break;
+                    }
+                }
+                // vcalendar 1.0 style;
+                if (!empty($params[$i]['EXPECT'])) {
+                    switch($params[$i]['EXPECT']) {
+                    case 'REQUEST':
+                        $attendance = KRONOLITH_PART_OPTIONAL;
+                        break;
+
+                    case 'FYI':
+                        $attendance = KRONOLITH_PART_NONE;
+                        break;
+                    }
+                }
+                $response = KRONOLITH_RESPONSE_NONE;
+                if (empty($params[$i]['PARTSTAT']) && !empty($params[$i]['STATUS'])) {
+                    $params[$i]['PARTSTAT']  = $params[$i]['STATUS'];
+                }
+
+                if (!empty($params[$i]['PARTSTAT'])) {
+                    switch($params[$i]['PARTSTAT']) {
+                    case 'ACCEPTED':
+                        $response = KRONOLITH_RESPONSE_ACCEPTED;
+                        break;
+
+                    case 'DECLINED':
+                        $response = KRONOLITH_RESPONSE_DECLINED;
+                        break;
+
+                    case 'TENTATIVE':
+                        $response = KRONOLITH_RESPONSE_TENTATIVE;
+                        break;
+                    }
+                }
+
+                $this->addAttendee($email, $attendance, $response);
+            }
+        }
+
         // Recurrence.
         $rrule = $vEvent->getAttribute('RRULE');
         if (!is_array($rrule) && !is_a($rrule, 'PEAR_Error') && strpos($rrule, '=') !== false) {
@@ -761,16 +855,24 @@ class Kronolith_Event {
                 }
             }
             if (count($time) == 3 && count($date) == 3) {
-                $this->start = &new Horde_Date(array('year' => $date[0], 'month' => $date[1], 'mday' => $date[2],
-                                                     'hour' => $time[0], 'min' => $time[1], 'sec' => $time[2]));
+                $this->start = &new Horde_Date(array('year' => $date[0],
+                                                     'month' => $date[1],
+                                                     'mday' => $date[2],
+                                                     'hour' => $time[0],
+                                                     'min' => $time[1],
+                                                     'sec' => $time[2]));
             }
         } else {
             return PEAR::raiseError(_("Events must have a start date."));
         }
-        if (empty($hash['duration']) && empty($hash['end_date'])) {
-            $hash['end_date'] = $hash['start_date'];
-        }
-        if (!empty($hash['duration'])) {
+        if (empty($hash['duration'])) {
+            if (empty($hash['end_date'])) {
+                $hash['end_date'] = $hash['start_date'];
+            }
+            if (empty($hash['end_time'])) {
+                $hash['end_time'] = $hash['start_time'];
+            }
+        } else {
             $weeks = str_replace('W', '', $hash['duration'][1]);
             $days = str_replace('D', '', $hash['duration'][2]);
             $hours = str_replace('H', '', $hash['duration'][4]);
@@ -790,8 +892,12 @@ class Kronolith_Event {
                 }
             }
             if (count($time) == 3 && count($date) == 3) {
-                $this->end = &new Horde_Date(array('year' => $date[0], 'month' => $date[1], 'mday' => $date[2],
-                                                   'hour' => $time[0], 'min' => $time[1], 'sec' => $time[2]));
+                $this->end = &new Horde_Date(array('year' => $date[0],
+                                                   'month' => $date[1],
+                                                   'mday' => $date[2],
+                                                   'hour' => $time[0],
+                                                   'min' => $time[1],
+                                                   'sec' => $time[2]));
             }
         }
         if (!empty($hash['alarm'])) {
@@ -1106,10 +1212,9 @@ class Kronolith_Event {
     }
 
     /**
-     * Set the category this event belongs to.
+     * Set the category of this event.
      *
-     * @param integer $category  The identifier of the category this event
-     *                           belongs to.
+     * @param string $category  The category of this event.
      */
     function setCategory($category)
     {
@@ -1117,9 +1222,9 @@ class Kronolith_Event {
     }
 
     /**
-     * Retrieve the category this event belongs to.
+     * Retrieve the category of this event.
      *
-     * @return integer  The identifier of the category this event belongs to.
+     * @return string  The category of this event.
      */
     function getCategory()
     {
@@ -1165,7 +1270,7 @@ class Kronolith_Event {
      */
     function hasStatus($status)
     {
-        return ($status === $this->status);
+        return ($status == $this->status);
     }
 
     /**
@@ -1186,7 +1291,7 @@ class Kronolith_Event {
      */
     function setAttendees($attendees)
     {
-        $this->attendees = $attendees;
+        $this->attendees = array_change_key_case($attendees);
     }
 
     /**
@@ -1199,6 +1304,7 @@ class Kronolith_Event {
      */
     function addAttendee($email, $attendance, $response)
     {
+        $email = String::lower($email);
         if ($attendance == KRONOLITH_PART_IGNORE) {
             if (isset($this->attendees[$email])) {
                 $attendance = $this->attendees[$email]['attendance'];
@@ -1220,6 +1326,7 @@ class Kronolith_Event {
      */
     function removeAttendee($email)
     {
+        $email = String::lower($email);
         if (isset($this->attendees[$email])) {
             unset($this->attendees[$email]);
         }
@@ -1246,6 +1353,7 @@ class Kronolith_Event {
      */
     function hasAttendee($email)
     {
+        $email = String::lower($email);
         return isset($this->attendees[$email]);
     }
 
@@ -1862,17 +1970,20 @@ class Kronolith_Event {
         global $prefs;
 
         $tooltip = '';
-
         if ($this->isAllDay()) {
-            $tooltip .= _("All day");
-        } elseif ($this->start->compareDate($this->end) != 0) {
-            $tooltip .= strftime($prefs->getValue('date_format'),
-                                 $this->start->timestamp()) . '-' .
-                strftime($prefs->getValue('date_format'),
-                         $this->end->timestamp());
+            $tooltip = _("All day");
+        } elseif (($cmp = $this->start->compareDate($this->end)) > 0) {
+            $df = $prefs->getValue('date_format');
+            if ($cmp > 0) {
+                $tooltip = strftime($df, $this->end->timestamp()) . '-' .
+                    strftime($df, $this->start->timestamp());
+            } else {
+                $tooltip = strftime($df, $this->start->timestamp()) . '-' .
+                    strftime($df, $this->end->timestamp());
+            }
         } else {
-            $tooltip .= date($prefs->getValue('twentyFour') ? 'G:i' : 'g:ia',
-                             $this->start->timestamp()) . '-' .
+            $tooltip = date($prefs->getValue('twentyFour') ? 'G:i' : 'g:ia',
+                            $this->start->timestamp()) . '-' .
                 date($prefs->getValue('twentyFour') ? 'G:i' : 'g:ia',
                      $this->end->timestamp());
         }
@@ -1922,7 +2033,7 @@ class Kronolith_Event {
         $after = &new Horde_Date($afterDate);
         $after->correct();
 
-        if ($this->start->compareDate($after) >= 0) {
+        if ($this->start->compareDateTime($after) >= 0) {
             return new Horde_Date($this->start);
         }
 
@@ -1930,21 +2041,15 @@ class Kronolith_Event {
             return false;
         }
 
-        if ($this->hasRecurEnd()) {
-            $this->recurEnd->hour = 23;
-            $this->recurEnd->min = 59;
-            $this->recurEnd->sec = 59;
-        }
-
         switch ($this->getRecurType()) {
         case KRONOLITH_RECUR_DAILY:
             $diff = Date_Calc::dateDiff($this->start->mday, $this->start->month, $this->start->year, $after->mday, $after->month, $after->year);
             $recur = ceil($diff / $this->recurInterval) * $this->recurInterval;
-            $next = $this->start;
+            $next = Util::cloneObject($this->start);
             list($next->mday, $next->month, $next->year) = explode('/', Date_Calc::daysToDate(Date_Calc::dateToDays($next->mday, $next->month, $next->year) + $recur, '%e/%m/%Y'));
             if ((!$this->hasRecurEnd() ||
-                 $next->compareDate($this->recurEnd) <= 0) &&
-                $next->compareDate($after) >= 0) {
+                 $next->compareDateTime($this->recurEnd) <= 0) &&
+                $next->compareDateTime($after) >= 0) {
                 return new Horde_Date($next);
             }
             break;
@@ -1969,22 +2074,24 @@ class Kronolith_Event {
             $next = $start_week;
             list($next->mday, $next->month, $next->year) = explode('/', Date_Calc::daysToDate(Date_Calc::dateToDays($next->mday, $next->month, $next->year) + $recur, '%e/%m/%Y'));
             $next = &new Horde_Date($next);
-            while ($next->compareDate($after) < 0 && $next->compareDate($after_week_end) < 0) {
+            while ($next->compareDateTime($after) < 0 &&
+                   $next->compareDateTime($after_week_end) < 0) {
                 ++$next->mday;
                 $next->correct();
             }
             if (!$this->hasRecurEnd() ||
-                $next->compareDate($this->recurEnd) <= 0) {
-                if ($next->compareDate($after_week_end) >= 0) {
+                $next->compareDateTime($this->recurEnd) <= 0) {
+                if ($next->compareDateTime($after_week_end) >= 0) {
                     return $this->nextRecurrence($after_week_end);
                 }
-                while (!$this->recurOnDay((int)pow(2, $next->dayOfWeek())) && $next->compareDate($after_week_end) < 0) {
+                while (!$this->recurOnDay((int)pow(2, $next->dayOfWeek())) &&
+                       $next->compareDateTime($after_week_end) < 0) {
                     ++$next->mday;
                     $next->correct();
                 }
                 if (!$this->hasRecurEnd() ||
-                    $next->compareDate($this->recurEnd) <= 0) {
-                    if ($next->compareDate($after_week_end) >= 0) {
+                    $next->compareDateTime($this->recurEnd) <= 0) {
+                    if ($next->compareDateTime($after_week_end) >= 0) {
                         return $this->nextRecurrence($after_week_end);
                     } else {
                         return $next;
@@ -1995,7 +2102,7 @@ class Kronolith_Event {
 
         case KRONOLITH_RECUR_DAY_OF_MONTH:
             $start = Util::cloneObject(new Horde_Date($this->start));
-            if ($after->compareDate($start) < 0) {
+            if ($after->compareDateTime($start) < 0) {
                 $after = $start;
             }
 
@@ -2036,7 +2143,7 @@ class Kronolith_Event {
 
                 // Bail if we've gone past the end of recurrence.
                 if ($this->hasRecurEnd() &&
-                    $this->recurEnd->compareDate($start) < 0) {
+                    $this->recurEnd->compareDateTime($start) < 0) {
                     return false;
                 }
             } while (true);
@@ -2065,13 +2172,13 @@ class Kronolith_Event {
                 $next = &Util::cloneObject($estart);
                 $next->setNthWeekday($weekday, $nth);
 
-                if ($next->compareDate($after) < 0) {
+                if ($next->compareDateTime($after) < 0) {
                     // We haven't made it past $after yet, try
                     // again.
                     continue;
                 }
                 if ($this->hasRecurEnd() &&
-                    $next->compareDate($this->recurEnd) > 0) {
+                    $next->compareDateTime($this->recurEnd) > 0) {
                     // We've gone past the end of recurrence; we can
                     // give up now.
                     return false;
@@ -2106,7 +2213,7 @@ class Kronolith_Event {
 
             // We've gone past the end of recurrence; give up.
             if ($this->hasRecurEnd() &&
-                $this->recurEnd->compareDate($estart) < 0) {
+                $this->recurEnd->compareDateTime($estart) < 0) {
                 return false;
             }
 
