@@ -7,7 +7,7 @@ require_once 'Horde/Identity.php';
  * Horde Kronolith driver for the Kolab IMAP Server.
  * Copyright 2004-2006 Horde Project (http://horde.org/)
  *
- * $Horde: kronolith/lib/Driver/kolab.php,v 1.16.2.8 2006/02/02 13:49:23 selsky Exp $
+ * $Horde: kronolith/lib/Driver/kolab.php,v 1.16.2.13 2006/08/25 13:21:21 jan Exp $
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -45,15 +45,15 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
 
     function _connect()
     {
-        if (!isset($this->_kolab)) {
-            $this->_kolab = &new Kolab();
+        if (!$this->_kolab) {
+            $this->_kolab = new Kolab();
         }
         return $this->_kolab->open($this->_calendar);
     }
 
     function _disconnect()
     {
-        if (empty($this->_kolab)) {
+        if (!$this->_kolab) {
             return true;
         }
 
@@ -65,6 +65,10 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
 
     function listAlarms($date)
     {
+        if (!$this->_kolab) {
+            return array();
+        }
+
         $allevents = $this->listEvents($date, $date, true);
         $events = array();
 
@@ -72,7 +76,7 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
             $event = &$this->getEvent($eventId);
 
             if ($event->getRecurType() == KRONOLITH_RECUR_NONE) {
-                $start = &new Horde_Date($event->start);
+                $start = new Horde_Date($event->start);
                 $start->min -= $event->getAlarm();
                 $start->correct();
                 if ($start->compareDateTime($date) <= 0 &&
@@ -81,7 +85,7 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
                 }
             } else {
                 if ($next = $this->nextRecurrence($eventId, $date)) {
-                    $start = &new Horde_Date($next);
+                    $start = new Horde_Date($next);
                     $start->min -= $event->getAlarm();
                     $start->correct();
                     if ($start->compareDateTime($date) <= 0 &&
@@ -90,7 +94,7 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
                                                                     'mday' => $next->mday,
                                                                     'hour' => $event->end->hour,
                                                                     'min' => $event->end->min,
-                                                                    'sec' => $event->end->sec)) <= -1)) {
+                                                                    'sec' => $event->end->sec))) <= -1) {
                         $events[] = $eventId;
                     }
                 }
@@ -150,7 +154,7 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
     function &getEvent($eventID = null)
     {
         if (is_null($eventID)) {
-            $event = &new Kronolith_Event_kolab($this);
+            $event = new Kronolith_Event_kolab($this);
             return $event;
         }
 
@@ -159,7 +163,7 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
             return $result;
         }
 
-        $event = &new Kronolith_Event_kolab($this);
+        $event = new Kronolith_Event_kolab($this);
         $event->fromDriver($this);
 
         return $event;
@@ -172,15 +176,13 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
         return $this->getEvent($uid);
     }
 
-    function saveEvent($event)
+    function saveEvent(&$event)
     {
-        $k = &$this->_kolab;
-
         $edit = false;
         if ($event->isStored() || $event->exists()) {
             $uid = $event->getID();
 
-            $result = $k->loadObject($uid);
+            $result = $this->_kolab->loadObject($uid);
             //No error check here, already done in exists()
 
             $edit = true;
@@ -189,84 +191,85 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
                 $uid = $event->getID();
             } else {
                 $uid = md5(uniqid(mt_rand(), true));
+                $event->setUID($uid);
             }
 
-            $k->newObject($uid);
+            $this->_kolab->newObject($uid);
         }
 
         $xml_hash = &$this->_kolab->getCurrentObject();
 
-        $k->setStr('summary', $event->getTitle());
-        $k->setStr('body', $event->getDescription());
-        $k->setStr('categories', $event->getCategory());
-        $k->setStr('location', $event->getLocation());
+        $this->_kolab->setStr('summary', $event->getTitle());
+        $this->_kolab->setStr('body', $event->getDescription());
+        $this->_kolab->setStr('categories', $event->getCategory());
+        $this->_kolab->setStr('location', $event->getLocation());
 
-        $organizer = &$k->initRootElem('organizer');
-        $k->setElemStr($organizer, 'smtp-address', $event->getCreatorID());
+        $organizer = &$this->_kolab->initRootElem('organizer');
+        $this->_kolab->setElemStr($organizer, 'smtp-address', $event->getCreatorID());
 
-        $k->setVal('alarm', $event->getAlarm());
+        $this->_kolab->setVal('alarm', $event->getAlarm());
         if ($event->isAllDay()) {
-            $k->setVal('start-date', Kolab::encodeDate($event->start->timestamp()));
-            $k->setVal('end-date', Kolab::encodeDate($event->end->timestamp()-24*60*60));
+            $this->_kolab->setVal('start-date', Kolab::encodeDate($event->start->timestamp()));
+            $this->_kolab->setVal('end-date', Kolab::encodeDate($event->end->timestamp()-24*60*60));
         } else {
-            $k->setVal('start-date', Kolab::encodeDateTime($event->start->timestamp()));
-            $k->setVal('end-date', Kolab::encodeDateTime($event->end->timestamp()));
+            $this->_kolab->setVal('start-date', Kolab::encodeDateTime($event->start->timestamp()));
+            $this->_kolab->setVal('end-date', Kolab::encodeDateTime($event->end->timestamp()));
         }
 
         switch ($event->status) {
         case KRONOLITH_STATUS_CANCELLED:
-            $k->setVal('show-time-as', 'free');
+            $this->_kolab->setVal('show-time-as', 'free');
             break;
 
         case KRONOLITH_STATUS_TENTATIVE:
-            $k->setVal('show-time-as', 'tentative');
+            $this->_kolab->setVal('show-time-as', 'tentative');
             break;
 
         case KRONOLITH_STATUS_CONFIRMED:
         default:
-            $k->setVal('show-time-as', 'busy');
+            $this->_kolab->setVal('show-time-as', 'busy');
             break;
         }
 
-        $k->delAllRootElems('attendee');
+        $this->_kolab->delAllRootElems('attendee');
         foreach ($event->attendees as $email => $status) {
-            $attendee = &$k->appendRootElem('attendee');
-            $k->setElemVal($attendee, 'smtp-address', $email);
+            $attendee = &$this->_kolab->appendRootElem('attendee');
+            $this->_kolab->setElemVal($attendee, 'smtp-address', $email);
 
             switch ($status['response']) {
             case KRONOLITH_RESPONSE_ACCEPTED:
-                $k->setElemVal($attendee, 'status', 'accepted');
+                $this->_kolab->setElemVal($attendee, 'status', 'accepted');
                 break;
 
             case KRONOLITH_RESPONSE_DECLINED:
-                $k->setElemVal($attendee, 'status', 'declined');
+                $this->_kolab->setElemVal($attendee, 'status', 'declined');
                 break;
 
             case KRONOLITH_RESPONSE_TENTATIVE:
-                $k->setElemVal($attendee, 'status', 'tentative');
+                $this->_kolab->setElemVal($attendee, 'status', 'tentative');
                 break;
 
             case KRONOLITH_RESPONSE_NONE:
             default:
-                $k->setElemVal($attendee, 'status', 'none');
+                $this->_kolab->setElemVal($attendee, 'status', 'none');
             }
 
             switch ($status['attendance']) {
             case KRONOLITH_PART_OPTIONAL:
-                $k->setElemVal($attendee, 'role', 'optional');
+                $this->_kolab->setElemVal($attendee, 'role', 'optional');
                 break;
 
             case KRONOLITH_PART_NONE:
-                $k->setElemVal($attendee, 'role', 'resource');
+                $this->_kolab->setElemVal($attendee, 'role', 'resource');
                 break;
 
             case KRONOLITH_PART_REQUIRED:
             default:
-                $k->setElemVal($attendee, 'role', 'required');
+                $this->_kolab->setElemVal($attendee, 'role', 'required');
             }
         }
 
-        $k->delAllRootElems('recurrence');
+        $this->_kolab->delAllRootElems('recurrence');
 
         $range_type = 'none';
         $range = 0;
@@ -276,9 +279,9 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
         }
 
         if ($event->recurType != KRONOLITH_RECUR_NONE) {
-            $recurrence = &$k->initRootElem('recurrence');
-            $k->setElemVal($recurrence, 'interval', $event->recurInterval);
-            $range = &$k->setElemVal($recurrence, 'range', $range);
+            $recurrence = &$this->_kolab->initRootElem('recurrence');
+            $this->_kolab->setElemVal($recurrence, 'interval', $event->recurInterval);
+            $range = &$this->_kolab->setElemVal($recurrence, 'range', $range);
             $range->set_attribute('type', $range_type);
 
             switch ($event->recurType) {
@@ -292,9 +295,9 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
                     $days = array('sunday', 'monday', 'tuesday', 'wednesday',
                                   'thursday', 'friday', 'saturday');
 
-                    for ($i = 0; $i <= 7 ; $i++) {
+                    for ($i = 0; $i <= 7 ; ++$i) {
                         if ($event->recurOnDay(pow(2, $i))) {
-                            $day = &$k->appendElem('day', $recurrence);
+                            $day = &$this->_kolab->appendElem('day', $recurrence);
                             $day->set_content($days[$i]);
                         }
                     }
@@ -303,17 +306,17 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
                 case KRONOLITH_RECUR_DAY_OF_MONTH:
                     $recurrence->set_attribute('cycle', 'monthly');
                     $recurrence->set_attribute('type', 'daynumber');
-                    $k->setElemVal($recurrence, 'date', $event->start->mday);
+                    $this->_kolab->setElemVal($recurrence, 'date', $event->start->mday);
                     break;
 
                 case KRONOLITH_RECUR_WEEK_OF_MONTH:
                     $recurrence->set_attribute('cycle', 'monthly');
                     $recurrence->set_attribute('type', 'weekday');
-                    $k->setElemVal($recurrence, 'daynumber', 1);
-                    $start = &new Horde_Date($event->start);
+                    $this->_kolab->setElemVal($recurrence, 'daynumber', 1);
+                    $start = new Horde_Date($event->start);
                     $days = array('sunday', 'monday', 'tuesday', 'wednesday',
                                   'thursday', 'friday', 'saturday');
-                    $k->setElemVal($recurrence, 'day', $days[$start->dayOfWeek()]);
+                    $this->_kolab->setElemVal($recurrence, 'day', $days[$start->dayOfWeek()]);
                     break;
 
                 case KRONOLITH_RECUR_YEARLY:
@@ -324,17 +327,21 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
                                     'may', 'june', 'july', 'august', 'september',
                                     'october', 'november', 'december');
 
-                    $k->setElemVal($recurrence, 'month', $months[$event->start->month]);
-                    $k->setElemVal($recurrence, 'date', $event->start->mday);
+                    $this->_kolab->setElemVal($recurrence, 'month', $months[$event->start->month]);
+                    $this->_kolab->setElemVal($recurrence, 'date', $event->start->mday);
                     break;
             }
         }
 
-        $result = $k->saveObject();
+        $result = $this->_kolab->saveObject();
         if (is_a($result, 'PEAR_Error')) {
             return $result;
         }
 
+        if (is_callable('Kolab', 'triggerFreeBusyUpdate')) {
+            Kolab::triggerFreeBusyUpdate($this->_calendar);
+        }
+
         /* Notify about the changed event. */
         $result = Kronolith::sendNotification($event, $edit ? 'edit' : 'add');
         if (is_a($result, 'PEAR_Error')) {
@@ -352,7 +359,14 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
      */
     function move($eventID, $newCalendar)
     {
-        return $this->_kolab->moveObject($eventID, $newCalendar);
+        $result = $this->_kolab->moveObject($eventID, $newCalendar);
+
+        if (is_callable('Kolab', 'triggerFreeBusyUpdate')) {
+            Kolab::triggerFreeBusyUpdate($this->_calendar);
+            Kolab::triggerFreeBusyUpdate($newCalendar);
+        }
+
+        return $result;
     }
 
     /**
@@ -400,6 +414,10 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
             return $deleted;
         }
 
+        if (is_callable('Kolab', 'triggerFreeBusyUpdate')) {
+            Kolab::triggerFreeBusyUpdate($this->_calendar);
+        }
+
         /* Notify about the deleted event. */
         $result = Kronolith::sendNotification($event, 'delete');
         if (is_a($result, 'PEAR_Error')) {
@@ -414,24 +432,24 @@ class Kronolith_Event_kolab extends Kronolith_Event {
     function fromDriver($dummy)
     {
         $driver = &$this->getDriver();
-        $k = &$driver->_kolab;
+        $kolab = &$driver->_kolab;
 
-        $this->eventID = $k->getUID();
-        $this->setUID($k->getUID());
-        $this->title = $k->getStr('summary');
-        $this->description = $k->getStr('body');
-        $this->location = $k->getStr('location');
-        $this->category = $k->getStr('categories');
+        $this->eventID = $kolab->getUID();
+        $this->setUID($kolab->getUID());
+        $this->title = $kolab->getStr('summary');
+        $this->description = $kolab->getStr('body');
+        $this->location = $kolab->getStr('location');
+        $this->category = $kolab->getStr('categories');
 
-        $organizer = &$k->getRootElem('organizer');
-        $this->creatorID = $k->getElemStr($organizer, 'smtp-address');
+        $organizer = &$kolab->getRootElem('organizer');
+        $this->creatorID = $kolab->getElemStr($organizer, 'smtp-address');
 
-        $this->alarm = $k->getVal('alarm');
-        $this->start = &new Horde_Date(Kolab::decodeDateOrDateTime($k->getVal('start-date')));
-        $this->end = &new Horde_Date(Kolab::decodeFullDayDate($k->getVal('end-date')));
+        $this->alarm = $kolab->getVal('alarm');
+        $this->start = new Horde_Date(Kolab::decodeDateOrDateTime($kolab->getVal('start-date')));
+        $this->end = new Horde_Date(Kolab::decodeFullDayDate($kolab->getVal('end-date')));
         $this->durMin = ($this->end->timestamp() - $this->start->timestamp()) / 60;
 
-        $status = $k->getVal('show-time-as');
+        $status = $kolab->getVal('show-time-as');
         switch ($status) {
         case 'free':
             $this->status = KRONOLITH_STATUS_CANCELLED;
@@ -447,16 +465,16 @@ class Kronolith_Event_kolab extends Kronolith_Event {
             $this->status = KRONOLITH_STATUS_CONFIRMED;
         }
 
-        $attendees = &$k->getAllRootElems('attendee');
-        for ($i = 0, $j = count($attendees); $i < $j; $i++) {
-            $attendee = &$attendees[$i];
+        $attendees = array_change_key_case($kolab->getAllRootElems('attendee'));
+        for ($i = 0, $iMax = count($attendees); $i < $iMax; ++$i) {
+            $attendee = $attendees[$i];
 
-            $email = $k->getElemStr($attendee, 'smtp-address');
+            $email = $kolab->getElemStr($attendee, 'smtp-address');
             if (empty($email)) {
                 continue;
             }
 
-            $role = $k->getElemVal($attendee, 'role');
+            $role = $kolab->getElemVal($attendee, 'role');
             switch ($role) {
             case 'optional':
                 $role = KRONOLITH_PART_OPTIONAL;
@@ -472,7 +490,7 @@ class Kronolith_Event_kolab extends Kronolith_Event {
                 break;
             }
 
-            $status = $k->getElemVal($attendee, 'status');
+            $status = $kolab->getElemVal($attendee, 'status');
             switch ($status) {
             case 'accepted':
                 $status = KRONOLITH_RESPONSE_ACCEPTED;
@@ -498,10 +516,10 @@ class Kronolith_Event_kolab extends Kronolith_Event {
         $this->recurEnd = null;
         $this->recurType = KRONOLITH_RECUR_NONE;
 
-        $recurrence = &$k->getRootElem('recurrence');
+        $recurrence = &$kolab->getRootElem('recurrence');
         if ($recurrence !== false) {
             $cycle = $recurrence->get_attribute('cycle');
-            $this->recurInterval = $k->getElemVal($recurrence, 'interval');
+            $this->recurInterval = $kolab->getElemVal($recurrence, 'interval');
 
             switch ($cycle) {
             case 'daily':
@@ -522,7 +540,7 @@ class Kronolith_Event_kolab extends Kronolith_Event {
                     'sunday' => HORDE_DATE_MASK_SUNDAY,
                 );
 
-                $days = $k->getAllElems('day', $recurrence);
+                $days = $kolab->getAllElems('day', $recurrence);
                 foreach ($days as $day) {
                     $day_str = $day->get_content();
 
@@ -554,9 +572,9 @@ class Kronolith_Event_kolab extends Kronolith_Event {
                 break;
             }
 
-            $range = &$k->getElem('range', $recurrence);
+            $range = &$kolab->getElem('range', $recurrence);
             $range_type = $range->get_attribute('type');
-            $range_val = $k->getElemVal($recurrence, 'range');
+            $range_val = $kolab->getElemVal($recurrence, 'range');
 
             switch ($range_type) {
             case 'number':
@@ -565,7 +583,7 @@ class Kronolith_Event_kolab extends Kronolith_Event {
                 break;
 
             case 'date':
-                $this->recurEnd = &new Horde_Date(Kolab::decodeDate($range_val));
+                $this->recurEnd = new Horde_Date(Kolab::decodeDate($range_val));
                 break;
             }
         }
