@@ -5,12 +5,12 @@ require_once 'Horde/Template.php';
 /**
  * This class represent a view of multiple free busy information sets.
  *
- * Copyright 2003-2005 Mike Cochrane <mike@graftonhall.co.nz>
- * Copyright 2004-2005 Jan Schneider <jan@horde.org>
+ * Copyright 2003-2006 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2004-2006 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file COPYING for license information.
  *
- * $Horde: kronolith/lib/FBView.php,v 1.10.10.4.2.1 2005/12/06 00:12:04 jan Exp $
+ * $Horde: kronolith/lib/FBView.php,v 1.10.10.9 2006/11/29 11:23:36 jan Exp $
  *
  * @author  Mike Cochrane <mike@graftonhall.co.nz>
  * @author  Jan Schneider <jan@horde.org>
@@ -24,7 +24,6 @@ class Kronolith_FreeBusy_View {
     var $_startHour;
     var $_endHour;
 
-    var $_day;
     var $_startStamp;
     var $_endStamp;
 
@@ -73,7 +72,8 @@ class Kronolith_FreeBusy_View {
         $template = &new Horde_Template();
         $template->set('title', $this->_title());
 
-        $html = $template->fetch(KRONOLITH_TEMPLATES . '/fbview/header.tpl');
+        $html = $template->fetch(KRONOLITH_TEMPLATES . '/fbview/header.tpl') .
+            '<div class="fbgrid">';
 
         $hours_html = $this->_hours();
 
@@ -95,8 +95,9 @@ class Kronolith_FreeBusy_View {
             }
 
             $template = &new Horde_Template();
-            $template->set('title', _("Required to attend"));
+            $template->set('title', _("Required Attendees"));
             $template->set('rows', $rows);
+            $template->set('span', count($this->_timeBlocks));
             $template->set('hours', $hours_html);
             $template->set('legend', '');
             $html .= $template->fetch(KRONOLITH_TEMPLATES . '/fbview/section.tpl');
@@ -115,14 +116,15 @@ class Kronolith_FreeBusy_View {
             }
 
             $template = &new Horde_Template();
-            $template->set('title', _("Optional to attend"));
+            $template->set('title', _("Optional Attendees"));
             $template->set('rows', $rows);
+            $template->set('span', count($this->_timeBlocks));
             $template->set('hours', $hours_html);
             $template->set('legend', '');
             $html .= $template->fetch(KRONOLITH_TEMPLATES . '/fbview/section.tpl');
         }
 
-        // possible meeting times.
+        // Possible meeting times.
         $optimal->setAttribute('ORGANIZER', _("All Attendees"));
         $blocks = $this->_getBlocks($optimal,
                                     $optimal->getFreePeriods($this->_startStamp, $this->_endStamp),
@@ -133,7 +135,7 @@ class Kronolith_FreeBusy_View {
         $template->set('blocks', $blocks);
         $rows = $template->fetch(KRONOLITH_TEMPLATES . '/fbview/row.tpl');
 
-        // possible meeting times.
+        // Possible meeting times.
         $required->setAttribute('ORGANIZER', _("Required Attendees"));
         $blocks = $this->_getBlocks($required,
                                     $required->getFreePeriods($this->_startStamp, $this->_endStamp),
@@ -144,22 +146,22 @@ class Kronolith_FreeBusy_View {
         $template->set('blocks', $blocks);
         $rows .= $template->fetch(KRONOLITH_TEMPLATES . '/fbview/row.tpl');
 
-        // reset locale
+        // Reset locale.
         setlocale(LC_NUMERIC, $lc);
 
         $template = &new Horde_Template();
-        $template->set('title', _("Possible Meeting Times"));
         $template->set('rows', $rows);
+        $template->set('title', _("Overview"));
+        $template->set('span', count($this->_timeBlocks));
         $template->set('hours', $hours_html);
-        $html .= $template->fetch(KRONOLITH_TEMPLATES . '/fbview/section.tpl');
-
         if ($prefs->getValue('show_fb_legend')) {
-            $legend_html = Util::bufferOutput('require', KRONOLITH_TEMPLATES . '/fbview/legend.inc');
+            $template->setOption('gettext', true);
+            $template->set('legend', $template->fetch(KRONOLITH_TEMPLATES . '/fbview/legend.tpl'));
         } else {
-            $legend_html = '';
+            $template->set('legend', '');
         }
 
-        return array($html, $legend_html);
+        return $html . $template->fetch(KRONOLITH_TEMPLATES . '/fbview/section.tpl') . '</div>';
     }
 
     /**
@@ -175,13 +177,15 @@ class Kronolith_FreeBusy_View {
     function &factory($view)
     {
         $driver = basename($view);
-        require_once dirname(__FILE__) . '/FBView/' . $view . '.php';
-        $class = 'Kronolith_FreeBusy_View_' . $view;
+        require_once dirname(__FILE__) . '/FBView/' . $driver . '.php';
+        $class = 'Kronolith_FreeBusy_View_' . $driver;
         if (class_exists($class)) {
-            return $ret = &new $class($user, $params);
+            $fbview = new $class($user, $params);
         } else {
-            return false;
+            $fbview = false;
         }
+
+        return $fbview;
     }
 
     /**
@@ -214,4 +218,33 @@ class Kronolith_FreeBusy_View {
         return $instances[$view];
     }
 
+    function _getBlocks($member, $periods, $blockfile, $label)
+    {
+        $template = &new Horde_Template();
+        $template->set('label', $label);
+
+        reset($periods);
+        list($periodStart, $periodEnd) = each($periods);
+
+        $blocks = '';
+        foreach ($this->_timeBlocks as $start => $end) {
+            if ($member->getStart() > $start ||
+                $member->getEnd() < $end) {
+                $blocks .= $template->fetch(KRONOLITH_TEMPLATES . '/fbview/unknownblock.tpl');
+                continue;
+            }
+
+            while ($end > $periodEnd &&
+                   list($periodStart, $periodEnd) = each($periods));
+
+            if ($start <= $periodEnd && $end >= $periodStart) {
+                $blocks .= $template->fetch(KRONOLITH_TEMPLATES . '/fbview/' . $blockfile);
+            } else {
+                $blocks .= $template->fetch(KRONOLITH_TEMPLATES . '/fbview/emptyblock.tpl');
+            }
+        }
+
+        return $blocks;
+    }
+
 }
