@@ -33,7 +33,7 @@ define('KRONOLITH_ERROR_FB_NOT_FOUND', 1);
 /**
  * The Kronolith:: class provides functionality common to all of Kronolith.
  *
- * $Horde: kronolith/lib/Kronolith.php,v 1.263.2.33 2006/02/20 22:16:13 jan Exp $
+ * $Horde: kronolith/lib/Kronolith.php,v 1.263.2.49 2006/10/24 09:16:37 jan Exp $
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @since   Kronolith 0.1
@@ -55,18 +55,21 @@ class Kronolith {
         global $kronolith;
 
         if (!isset($startDate)) {
-            $startDate = &new Horde_Date(time());
+            $startDate = new Horde_Date(time());
         } else {
             $startDate = Util::cloneObject(new Horde_Date($startDate));
         }
         if (!isset($endDate)) {
-            $endDate = &new Horde_Date(time());
+            $endDate = new Horde_Date(time());
         } else {
             $endDate = Util::cloneObject(new Horde_Date($endDate));
         }
         if (!isset($calendars)) {
             $calendars = $GLOBALS['display_calendars'];
         }
+        if (!is_array($calendars)) {
+            $calendars = array($calendars);
+        }
 
         $eventIds = array();
         foreach ($calendars as $cal) {
@@ -143,6 +146,8 @@ class Kronolith {
      */
     function getRemoteCalendar($url)
     {
+        $url = trim($url);
+
         // Treat webcal:// URLs as http://.
         if (substr($url, 0, 9) == 'webcal://') {
             $url = str_replace('webcal://', 'http://', $url);
@@ -154,7 +159,7 @@ class Kronolith {
             $options['allowRedirects'] = true;
 
             require_once 'HTTP/Request.php';
-            $http = &new HTTP_Request($url, $options);
+            $http = new HTTP_Request($url, $options);
             @$http->sendRequest();
             if ($http->getResponseCode() != 200) {
                 Horde::logMessage(sprintf('Failed to retrieve remote calendar: url = "%s", status = %s',
@@ -188,7 +193,7 @@ class Kronolith {
         }
 
         require_once 'Horde/iCalendar.php';
-        $iCal = &new Horde_iCalendar();
+        $iCal = new Horde_iCalendar();
         if (!$iCal->parsevCalendar($data)) {
             return array();
         }
@@ -200,6 +205,7 @@ class Kronolith {
             $component = $components[$i];
             if ($component->getType() == 'vEvent') {
                 $event = &$kronolith->getEvent();
+                $event->status = KRONOLITH_STATUS_FREE;
                 $event->fromiCalendar($component);
                 $event->remoteCal = $url;
                 $event->eventID = $i;
@@ -231,7 +237,7 @@ class Kronolith {
         }
 
         require_once 'Horde/iCalendar.php';
-        $iCal = &new Horde_iCalendar();
+        $iCal = new Horde_iCalendar();
         if (!$iCal->parsevCalendar($data)) {
             return array();
         }
@@ -240,6 +246,7 @@ class Kronolith {
         if (isset($components[$eventId]) &&
             $components[$eventId]->getType() == 'vEvent') {
             $event = &$kronolith->getEvent();
+            $event->status = KRONOLITH_STATUS_FREE;
             $event->fromiCalendar($components[$eventId]);
             $event->remoteCal = $url;
             $event->eventID = $eventId;
@@ -269,12 +276,12 @@ class Kronolith {
         global $kronolith, $prefs, $registry;
 
         if (!isset($startDate)) {
-            $startDate = &new Horde_Date(time());
+            $startDate = new Horde_Date(time());
         } else {
             $startDate = Util::cloneObject(new Horde_Date($startDate));
         }
         if (!isset($endDate)) {
-            $endDate = &new Horde_Date(time());
+            $endDate = new Horde_Date(time());
         } else {
             $endDate = Util::cloneObject(new Horde_Date($endDate));
         }
@@ -330,8 +337,8 @@ class Kronolith {
                             isset($task['category'])) {
                             $event->category = isset($task['category']) ? $task['category'] : null;
                         }
-                        $event->start = &new Horde_Date($task['due']);
-                        $event->end = &new Horde_Date($task['due'] + 1);
+                        $event->start = new Horde_Date($task['due']);
+                        $event->end = new Horde_Date($task['due'] + 1);
                         $dayStamp = mktime(0, 0, 0,
                                            date('n', $task['due']),
                                            date('j', $task['due']),
@@ -379,6 +386,14 @@ class Kronolith {
         if (!$event->hasRecurType(KRONOLITH_RECUR_NONE) && $showRecurrence) {
             // Recurring Event.
 
+            /* We can't use the event duration here because we might cover a
+             * daylight saving time switch. */
+            $diff = array($event->end->year - $event->start->year,
+                          $event->end->month - $event->start->month,
+                          $event->end->mday - $event->start->mday,
+                          $event->end->hour - $event->start->hour,
+                          $event->end->min - $event->start->min);
+
             if ($event->start->compareDateTime($startOfPeriod) < 0) {
                 // The first time the event happens was before the period
                 // started. Start searching for recurrences from the start of
@@ -392,10 +407,7 @@ class Kronolith {
                 if (!$event->hasException($event->start->year,
                                           $event->start->month,
                                           $event->start->mday)) {
-                    $results[mktime(0, 0, 0,
-                                    $event->start->month,
-                                    $event->start->mday,
-                                    $event->start->year)][$event->getID()] = $event;
+                    Kronolith::_addCoverDates($results, $event, $event->start, $event->end);
                 }
 
                 // Start searching for recurrences from the day after it
@@ -409,7 +421,15 @@ class Kronolith {
             $next = $event->nextRecurrence($next);
             while ($next !== false && $next->compareDate($endDate) <= 0) {
                 if (!$event->hasException($next->year, $next->month, $next->mday)) {
-                    $results[$next->datestamp()][$event->getID()] = $event;
+                    /* Add the event to all the days it covers. */
+                    $nextEnd = Util::cloneObject($next);
+                    $nextEnd->year  += $diff[0];
+                    $nextEnd->month += $diff[1];
+                    $nextEnd->mday  += $diff[2];
+                    $nextEnd->hour  += $diff[3];
+                    $nextEnd->min   += $diff[4];
+                    $nextEnd->correct();
+                    Kronolith::_addCoverDates($results, $event, $next, $nextEnd);
                 }
                 $next = $event->nextRecurrence(array('year' => $next->year,
                                                      'month' => $next->month,
@@ -432,7 +452,7 @@ class Kronolith {
             // Work out what day it ends on.
             if ($event->end->compareDateTime($endOfPeriod) > 0) {
                 // Ends after the end of the period.
-                $eventEnd = Util::cloneObject($endOfPeriod);
+                $eventEnd = Util::cloneObject($event->end);
             } else {
                 // If the event doesn't end at 12am set the end date to the
                 // current end date. If it ends at 12am and does not end at
@@ -440,35 +460,53 @@ class Kronolith {
                 // to the previous day's end date.
                 if ($event->end->hour != 0 ||
                     $event->end->min != 0 ||
-                    $event->start->compareDateTime($event->end) == 0) {
+                    $event->start->compareDateTime($event->end) == 0 ||
+                    $event->isAllDay()) {
                     $eventEnd = Util::cloneObject($event->end);
                 } else {
-                    $eventEnd = &new Horde_Date(
-                        array('hour' => 23,
-                              'min' => 59,
-                              'sec' => 59,
-                              'month' =>$event->end->month,
-                              'mday' => $event->end->mday - 1,
-                              'year' => $event->end->year));
+                    $eventEnd = new Horde_Date(
+                        array('hour' =>  23,
+                              'min' =>   59,
+                              'sec' =>   59,
+                              'month' => $event->end->month,
+                              'mday' =>  $event->end->mday - 1,
+                              'year' =>  $event->end->year));
                 }
             }
 
             // Add the event to all the days it covers.
-            $i = $eventStart->mday;
-            $loopDate = &new Horde_Date(array('month' => $eventStart->month,
-                                              'mday' => $i,
-                                              'year' => $eventStart->year));
-            while ($loopDate->compareDateTime($eventEnd) <= 0) {
-                if (!($event->isAllDay() &&
-                      $loopDate->compareDateTime($eventEnd) == 0)) {
-                    $results[$loopDate->timestamp()][$event->getID()] = $event;
-                }
-                $loopDate = &new Horde_Date(
-                    array('month' => $eventStart->month,
-                          'mday' => ++$i,
-                          'year' => $eventStart->year));
-                $loopDate->correct();
+            Kronolith::_addCoverDates($results, $event, $eventStart, $eventEnd);
+        }
+    }
+
+    /**
+     * Adds an event to all the days it covers.
+     *
+     * @param array $result           The current result list.
+     * @param Kronolith_Event $event  An event object.
+     * @param Horde_Date $eventStart  The event's start at the actual
+     *                                recurrence.
+     * @param Horde_Date $eventEnd    The event's end at the actual recurrence.
+     */
+    function _addCoverDates(&$results, $event, $eventStart, $eventEnd)
+    {
+        $i = $eventStart->mday;
+        $loopDate = new Horde_Date(array('month' => $eventStart->month,
+                                         'mday' => $i,
+                                         'year' => $eventStart->year));
+        while ($loopDate->compareDateTime($eventEnd) <= 0) {
+            if (!$event->isAllDay() ||
+                $loopDate->compareDateTime($eventEnd) != 0) {
+                $addEvent = Util::cloneObject($event);
+                $addEvent->start = $eventStart;
+                $addEvent->end = $eventEnd;
+                $results[$loopDate->timestamp()][$addEvent->getID()] = $addEvent;
             }
+            $loopDate = new Horde_Date(
+                array('month' => $eventStart->month,
+                      'mday' => ++$i,
+                      'year' => $eventStart->year));
+            $loopDate->correct();
         }
     }
 
@@ -510,7 +548,7 @@ class Kronolith {
     }
 
     /**
-     * Returns the real name, if available, of an event's/calendar's owner.
+     * Returns the real name, if available, of a user.
      */
     function getUserName($uid)
     {
@@ -529,6 +567,26 @@ class Kronolith {
         return $names[$uid];
     }
 
+    /**
+     * Returns the email address, if available, of a user.
+     */
+    function getUserEmail($uid)
+    {
+        static $emails = array();
+
+        if (!isset($emails[$uid])) {
+            require_once 'Horde/Identity.php';
+            $ident = &Identity::singleton('none', $uid);
+            $ident->setDefault($ident->getDefault());
+            $emails[$uid] = $ident->getValue('from_addr');
+            if (empty($emails[$uid])) {
+                $emails[$uid] = $uid;
+            }
+        }
+
+        return $emails[$uid];
+    }
+
     /**
      * Maps a Kronolith recurrence value to a translated string suitable for
      * display.
@@ -709,6 +767,9 @@ class Kronolith {
      * Returns all calendars a user has access to, according to several
      * parameters/permission levels.
      *
+     * This method doesn't work (obviously) if no DataTree backend has been
+     * configured.
+     *
      * @param boolean $owneronly   Only return calenders that this user owns?
      *                             Defaults to false.
      * @param integer $permission  The permission to filter calendars by.
@@ -737,10 +798,12 @@ class Kronolith {
         $default_share = $prefs->getValue('default_share');
         $calendars = Kronolith::listCalendars(false, $permission);
 
-        if (isset($calendars[$default_share])) {
+        if (isset($calendars[$default_share]) ||
+            $prefs->isLocked('default_share')) {
             return $default_share;
-        } elseif ($prefs->isLocked('default_share')) {
-            return '';
+        } elseif (isset($GLOBALS['all_calendars'][Auth::getAuth()]) &&
+                  $GLOBALS['all_calendars'][Auth::getAuth()]->hasPermission(Auth::getAuth(), $permission)) {
+            return Auth::getAuth();
         } elseif (count($calendars)) {
             return key($calendars);
         }
@@ -776,7 +839,7 @@ class Kronolith {
         // Fetch the appropriate share and check permissions.
         $share = &$kronolith_shares->getShare($calendar[0]);
         if (is_a($share, 'PEAR_Error')) {
-            return '';
+            return $returnObj ? $share : '';
         }
 
         // Default the start date to today.
@@ -806,12 +869,12 @@ class Kronolith {
         $cn = $identity->getValue('fullname');
 
         // Fetch events.
-        $startDate = &new Horde_Date($startstamp);
-        $endDate = &new Horde_Date($endstamp);
+        $startDate = new Horde_Date($startstamp);
+        $endDate = new Horde_Date($endstamp);
         $busy = Kronolith::listEvents($startDate, $endDate, $calendar);
 
         // Create the new iCalendar.
-        $vCal = &new Horde_iCalendar();
+        $vCal = new Horde_iCalendar();
         $vCal->setAttribute('PRODID', '-//The Horde Project//Kronolith ' . KRONOLITH_VERSION . '//EN');
         $vCal->setAttribute('METHOD', 'PUBLISH');
 
@@ -890,7 +953,7 @@ class Kronolith {
         // Properly handle RFC822-compliant email addresses.
         static $rfc822;
         if (is_null($rfc822)) {
-            $rfc822 = &new Mail_RFC822();
+            $rfc822 = new Mail_RFC822();
         }
 
         $default_domain = empty($conf['storage']['default_domain']) ? null : $conf['storage']['default_domain'];
@@ -908,17 +971,20 @@ class Kronolith {
         // set.
         $url = Kronolith::getFreeBusyUrl($email);
         if (is_a($url, 'PEAR_Error')) {
-            unset($url);
+            $url = null;
+        } else {
+            $url = trim($url);
         }
-        if (!empty($url)) {
+        if ($url) {
             require_once 'HTTP/Request.php';
-            $http = &new HTTP_Request($url, array('method' => 'GET',
-                                                  'timeout' => 5,
-                                                  'allowRedirects' => true));
+            $http = new HTTP_Request($url,
+                                     array('method' => 'GET',
+                                           'timeout' => 5,
+                                           'allowRedirects' => true));
             @$http->sendRequest();
             if ($http->getResponseCode() == 200 &&
                 $data = $http->getResponseBody()) {
-                $vCal = &new Horde_iCalendar();
+                $vCal = new Horde_iCalendar();
                 $vCal->parsevCalendar($data);
 
                 $vFb = &$vCal->findComponent('VFREEBUSY');
@@ -934,12 +1000,16 @@ class Kronolith {
         $storage = &Kronolith_Storage::singleton();
 
         $fb = $storage->search($email);
-        if (!is_a($fb, 'PEAR_Error') || $fb->getCode() == KRONOLITH_ERROR_FB_NOT_FOUND) {
+        if (!is_a($fb, 'PEAR_Error')) {
             return $fb;
+        } elseif ($fb->getCode() == KRONOLITH_ERROR_FB_NOT_FOUND) {
+            return $url
+                ? PEAR::raiseError(sprintf(_("No free/busy information found at the free/busy url of %s."), $email))
+                : PEAR::raiseError(sprintf(_("No free/busy url found for %s."), $email));
         }
 
         // Or else return an empty VFB object
-        $vCal = &new Horde_iCalendar();
+        $vCal = new Horde_iCalendar();
         $vFb = &Horde_iCalendar::newComponent('vfreebusy', $vCal);
         $vFb->setAttribute('ORGANIZER', $email);
 
@@ -1023,13 +1093,14 @@ class Kronolith {
 
             case KRONOLITH_ITIP_REQUEST:
             default:
-                $method = 'REQUEST';
                 if ($status['response'] == KRONOLITH_RESPONSE_NONE) {
                     // Invitation.
+                    $method = 'REQUEST';
                     $filename = 'event-invitation.ics';
                     $subject = $event->getTitle();
                 } else {
                     // Update.
+                    $method = 'ADD';
                     $filename = 'event-update.ics';
                     $subject = sprintf(_("Updated: %s."), $event->getTitle());
                 }
@@ -1050,17 +1121,17 @@ class Kronolith {
                 $message .= "\n\n" . sprintf(_("If your email client doesn't support iTip requests you can use one of the following links to accept or decline the event.\n\nTo accept the event:\n%s\n\nTo accept the event tentatively:\n%s\n\nTo decline the event:\n%s"), Util::addParameter($attend_link, 'a', 'accept', false), Util::addParameter($attend_link, 'a', 'tentative', false), Util::addParameter($attend_link, 'a', 'decline', false));
             }
 
-            $mime = &new MIME_Part('multipart/alternative');
-            $body = &new MIME_Part('text/plain', String::wrap($message, 80, "\n"));
-            $body->setCharset(NLS::getCharset());
+            $mime = new MIME_Part('multipart/alternative');
+            $body = new MIME_Part('text/plain', $message, NLS::getCharset());
+            $body->setTransferEncoding('quoted-printable');
 
             require_once 'Horde/Data.php';
             require_once 'Horde/iCalendar.php';
 
-            $iCal = &new Horde_iCalendar();
+            $iCal = new Horde_iCalendar();
             $iCal->setAttribute('METHOD', $method);
-            $iCal->addComponent($event->toiCalendar($iCal, $ident));
-            $ics = &new MIME_Part('text/calendar', $iCal->exportvCalendar());
+            $iCal->addComponent($event->toiCalendar($iCal));
+            $ics = new MIME_Part('text/calendar', $iCal->exportvCalendar());
             $ics->setName($filename);
             $ics->setContentTypeParameter('METHOD', $method);
             $ics->setCharset(NLS::getCharset());
@@ -1070,7 +1141,7 @@ class Kronolith {
             $mime = &MIME_Message::convertMimePart($mime);
 
             // Build the notification headers.
-            $msg_headers = &new MIME_Headers();
+            $msg_headers = new MIME_Headers();
             $msg_headers->addReceivedHeader();
             $msg_headers->addMessageIdHeader();
             $msg_headers->addHeader('Date', date('r'));
@@ -1154,6 +1225,9 @@ class Kronolith {
         }
         foreach ($share->listGroups(PERMS_READ) as $group) {
             $group = $groups->getGroupById($group);
+            if (is_a($group, 'PEAR_Error')) {
+                continue;
+            }
             $group_users = $group->listAllUsers();
             if (is_a($group_users, 'PEAR_Error')) {
                 Horde::logMessage($group_users, __FILE__, __LINE__, PEAR_LOG_ERR);
@@ -1182,7 +1256,7 @@ class Kronolith {
             return;
         }
 
-        $msg_headers = &new MIME_Headers();
+        $msg_headers = new MIME_Headers();
         $msg_headers->addMessageIdHeader();
         $msg_headers->addAgentHeader();
         $msg_headers->addHeader('Date', date('r'));
@@ -1191,8 +1265,8 @@ class Kronolith {
 
         $message = "\n" . sprintf($notification_message, $event->title, $share->get('name'), strftime('%x', $event->start->timestamp()), date('H:i', $event->start->timestamp())) . "\n\n" . $event->getDescription();
 
-        $mime = &new MIME_Message();
-        $body = &new MIME_Part('text/plain', String::wrap($message, 76, "\n"), NLS::getCharset());
+        $mime = new MIME_Message();
+        $body = new MIME_Part('text/plain', String::wrap($message, 76, "\n"), NLS::getCharset());
 
         $mime->addPart($body);
         $msg_headers->addMIMEHeaders($mime);
@@ -1305,7 +1379,7 @@ class Kronolith {
     {
         require_once 'Horde/UI/Tabs.php';
         require_once 'Horde/Variables.php';
-        $tabs = &new Horde_UI_Tabs('view', Variables::getDefaultVariables());
+        $tabs = new Horde_UI_Tabs('view', Variables::getDefaultVariables());
         $tabs->preserve('timestamp', Kronolith::currentTimestamp());
 
         $tabs->addTab(_("Day"), Horde::applicationUrl('day.php'), 'day');
@@ -1331,7 +1405,7 @@ class Kronolith {
         }
 
         require_once 'Horde/Menu.php';
-        $menu = &new Menu();
+        $menu = new Menu();
 
         if (Auth::getAuth()) {
             $menu->add(Horde::applicationUrl('calendars.php'), _("_My Calendars"), 'calendars.png');
@@ -1346,7 +1420,7 @@ class Kronolith {
 
         $menu->add(Horde::applicationUrl($prefs->getValue('defaultview') . '.php'), _("_Today"), 'today.png', null, null, null, '__noselection');
         if ($browser->hasFeature('dom')) {
-            Horde::addScriptFile('goto.inc', 'kronolith');
+            Horde::addScriptFile('goto.js', 'kronolith');
             $menu->add('#', _("_Goto"), 'goto.png', null, '', 'openKGoto(\'' . Kronolith::currentTimestamp() . '\'); return false;');
         }
         $menu->add(Horde::applicationUrl('search.php'), _("_Search"), 'search.png', $registry->getImageDir('horde'));
