@@ -4,7 +4,7 @@
  *
  * Act on alarms in events and send emails/pages/etc. to users.
  *
- * $Horde: kronolith/lib/Scheduler/kronolith.php,v 1.25.6.10 2006/02/19 23:56:48 chuck Exp $
+ * $Horde: kronolith/lib/Scheduler/kronolith.php,v 1.25.6.13 2006/10/20 15:38:24 jan Exp $
  *
  * @package Horde_Scheduler
  */
@@ -79,16 +79,18 @@ class Horde_Scheduler_kronolith extends Horde_Scheduler {
                     continue;
                 }
 
-                /* Set the event's start date to the next recurrence
-                 * date. This should avoid problems when an alarm
-                 * triggers on a different day from the actual event,
-                 * and make $seenid unique for each occurrence of a
-                 * recurring event. */
-                $event->start = $event->nextRecurrence($this->_runtime);
+                if ($event->getRecurType() != KRONOLITH_RECUR_NONE) {
+                    /* Set the event's start date to the next
+                     * recurrence date. This should avoid problems
+                     * when an alarm triggers on a different day from
+                     * the actual event, and make $seenid unique for
+                     * each occurrence of a recurring event. */
+                    $event->start = $event->nextRecurrence($this->_runtime);
 
-                /* Check for exceptions; do nothing if one is found. */
-                if ($event->hasException($event->start->year, $event->start->month, $event->start->mday)) {
-                    continue;
+                    /* Check for exceptions; do nothing if one is found. */
+                    if ($event->hasException($event->start->year, $event->start->month, $event->start->mday)) {
+                        continue;
+                    }
                 }
 
                 $seenid = $eventId . $event->start->timestamp() . $event->getAlarm();
@@ -143,7 +145,7 @@ class Horde_Scheduler_kronolith extends Horde_Scheduler {
 
             if (!empty($emails[$user])) {
                 $prefs = &Prefs::singleton($conf['prefs']['driver'],
-                                       'kronolith', $user);
+                                           'kronolith', $user);
                 $prefs->retrieve();
                 $shown_calendars = unserialize($prefs->getValue('display_cals'));
                 $reminder = $prefs->getValue('event_reminder');
@@ -198,8 +200,8 @@ class Horde_Scheduler_kronolith extends Horde_Scheduler {
         $mail_params = $conf['mailer']['params'];
         if ($mail_driver == 'smtp' && $mail_params['auth'] &&
             empty($mail_params['username'])) {
-            $mail_params['username'] = Auth::getAuth();
-            $mail_params['password'] = Auth::getCredential('password');
+            Horde::logMessage('Reminders don\'t work with user based SMTP authentication.', __FILE__, __LINE__, PEAR_LOG_ERR);
+            return;
         }
 
         foreach ($recipients as $lang => $lang_recipients) {
