@@ -2,7 +2,7 @@
 /**
  * The Kronolith_WeekView:: class provides an API for viewing weeks.
  *
- * $Horde: kronolith/lib/WeekView.php,v 1.88.2.3 2005/07/03 03:16:35 selsky Exp $
+ * $Horde: kronolith/lib/WeekView.php,v 1.88.2.4 2005/10/18 12:27:30 jan Exp $
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jan Schneider <jan@horde.org>
@@ -20,6 +20,21 @@ class Kronolith_WeekView {
     var $_sidebyside = false;
     var $_currentCalendars = array();
 
+    /**
+     * How many time slots are we dividing each hour into?
+     *
+     * @var integer
+     */
+    var $_slotsPerHour = 2;
+
+    /**
+     * How many slots do we have per day? Calculated from $_slotsPerHour.
+     *
+     * @see $_slotsPerHour
+     * @var integer
+     */
+    var $_slotsPerDay;
+
     function Kronolith_WeekView($week = null, $year = null, $startDay = null, $endDay = null)
     {
         if (empty($year)) {
@@ -74,11 +89,14 @@ class Kronolith_WeekView {
         }
         $this->_sidebyside = $this->days[$this->startDay]->_sidebyside;
         $this->_currentCalendars = $this->days[$this->startDay]->_currentCalendars;
+        $this->_slotsPerHour = $this->days[$this->startDay]->_slotsPerHour;
+        $this->_slotsPerDay = $this->days[$this->startDay]->_slotsPerDay;
+        $this->_slotLength = $this->days[$this->startDay]->_slotLength;
     }
 
     function html($template_path = KRONOLITH_TEMPLATES)
     {
-        global $prefs, $print_view, $cManager;
+        global $prefs, $print_view;
 
         $more_timeslots = $prefs->getValue('time_between_days');
         $include_all_events = !$prefs->getValue('show_shared_side_by_side');
@@ -87,10 +105,11 @@ class Kronolith_WeekView {
             $this->parse();
         }
 
-        $hours = $this->days[$this->startDay]->hours;
+        $slots = $this->days[$this->startDay]->slots;
         $cid = 0;
         require $template_path . '/week/head.inc';
         if ($this->_sidebyside) {
+            require $template_path . '/javascript/open_calendar_search.js';
             require $template_path . '/week/head_side_by_side.inc';
         }
 
@@ -107,31 +126,27 @@ class Kronolith_WeekView {
         } else {
             $addeventurl = _("All day");
         }
-        $rowspan = ' rowspan="1"';
 
-        $first_row = true;
+        $eventCategories = array();
+
         $row = '';
-
-        $colors = $cManager->colors();
-
         for ($j = $this->startDay; $j <= $this->endDay; $j++) {
             $row .= '<td class="hour" align="right">' . ($more_timeslots ? _("All day") : '&nbsp;') . '</td>';
-            $row .= '<td colspan="' . $this->days[$j]->_totalspan . '" valign="top" style="padding:0px"><table width="100%" cellspacing="0">';
+            $row .= '<td colspan="' . $this->days[$j]->_totalspan . '" valign="top"><table width="100%" cellspacing="0">';
             if ($this->days[$j]->_all_day_maxrowspan > 0) {
                 for ($k = 0; $k < $this->days[$j]->_all_day_maxrowspan; $k++) {
                     $row .= '<tr>';
                     foreach ($this->days[$j]->_currentCalendars as $cid => $cal) {
                         if (count($this->days[$j]->_all_day_events[$cid]) === $k) {
-                            $row .= '<td rowspan="' . ($this->days[$j]->_all_day_maxrowspan - $k) . '" width="'. round(99/count($this->days[$j]->_currentCalendars)) . '%">&nbsp;</td>';
+                            $row .= '<td rowspan="' . ($this->days[$j]->_all_day_maxrowspan - $k) . '" width="'. round(99 / count($this->days[$j]->_currentCalendars)) . '%">&nbsp;</td>';
                         } elseif (count($this->days[$j]->_all_day_events[$cid]) > $k) {
                             $event = $this->days[$j]->_all_day_events[$cid][$k];
-                            $categoryColor = isset($colors[$event->getCategory()]) ? $colors[$event->getCategory()] : '#ccffcc';
-                            $row .= '<td class="week-eventbox" style="background-color: ' . $categoryColor . '; ';
-                            $row .= 'border-color:' . Kronolith::borderColor($categoryColor) . ';" ';
-                            $row .= 'width="' . round(99 / count($this->days[$j]->_currentCalendars)) . '%" ';
-                            $row .= 'valign="top">';
-                            $row .= $event->getLink($this->days[$j]->getStamp());
-                            $row .= '</td>';
+                            $eventCategories[$event->getCategory()] = true;
+
+                            $row .= '<td class="week-eventbox category' . md5($event->getCategory()) . '" ' .
+                                'width="' . round(99 / count($this->days[$j]->_currentCalendars)) . '%" ' .
+                                'valign="top">' .
+                                $event->getLink($this->days[$j]->getStamp()) . '</td>';
                         }
                     }
                     $row .= '</tr>';
@@ -142,44 +157,38 @@ class Kronolith_WeekView {
             $row .= '</table></td>';
         }
 
+        $rowspan = '';
+        $first_row = true;
         require $template_path . '/day/all_day.inc';
 
-        $addeventurl = null;
-        $rows = $eventCategories = array();
-        for ($i = 0; $i < 48; $i++) {
-            if ($i >= $prefs->getValue('day_hour_end') && $i > $this->last) {
+        $twenty_four = $prefs->getValue('twentyFour');
+        $day_hour_start = $prefs->getValue('day_hour_start') / 2 * $this->_slotsPerHour;
+        $day_hour_end = $prefs->getValue('day_hour_end') / 2 * $this->_slotsPerHour;
+        $rows = array();
+
+        for ($i = 0; $i < $this->_slotsPerDay; $i++) {
+            if ($i >= $day_hour_end && $i > $this->last) {
                 break;
             }
-            if ($i < $this->first && $i < $prefs->getValue('day_hour_start')) {
+            if ($i < $this->first && $i < $day_hour_start) {
                 continue;
             }
 
-            if ($prefs->getValue('half') && $i % 2 != 0) {
-                $time = ':30';
+            if (($m = $i % $this->_slotsPerHour) != 0) {
+                $time = ':' . $m * $this->_slotLength;
                 $hourclass = 'halfhour';
             } else {
-                $time = date(($prefs->getValue('twentyFour')) ? 'G' : 'ga', $hours[$i]['timestamp']);
+                $time = date($twenty_four ? 'G' : 'ga', $slots[$i]['timestamp']);
                 $hourclass = 'hour';
             }
-            $style = ((($prefs->getValue('half')) ? $i : floor($i / 2)) % 2) ? 'item1' : 'item0';
 
             $row = '';
             for ($j = $this->startDay; $j <= $this->endDay; $j++) {
                 // Add spacer between days, or timeslots.
                 if ($more_timeslots) {
-                    if ($prefs->getValue('half')) {
-                        if ($i % 2 == 0) {
-                            $row .= '<td align="right" class="hour">' . $time . '</td>';
-                        } else {
-                            $row .= '<td align="right" class="halfhour">' . $time . '</td>';
-                        }
-                    } else {
-                        if ($i % 2 == 0) {
-                            $row .= '<td rowspan="2" align="right" class="hour">' . $time . '</td>';
-                        }
-                    }
+                    $row .= '<td align="right" class="' . $hourclass . '">' . $time . '</td>';
                 } else {
-                    $row .= '<td width="1%" class="text">&nbsp;</td>';
+                    $row .= '<td width="1%">&nbsp;</td>';
                 }
 
                 if (!count($this->_currentCalendars)) {
@@ -198,17 +207,16 @@ class Kronolith_WeekView {
                             $span = $this->days[$j]->_span[$cid] / $event->overlap;
                             $hspan += $span;
 
-                            $start = mktime(floor($i / 2), ($i % 2) * 30, 0, $this->days[$j]->month, $this->days[$j]->mday, $this->days[$j]->year);
-                            if (($event->start->timestamp() >= $start && $event->start->timestamp() < $start + 60 * 30) || $start == $this->days[$j]->getStamp()) {
-                                $categoryColor = isset($colors[$event->getCategory()]) ? $colors[$event->getCategory()] : $colors['_default_'];
+                            $start = mktime(floor($i / $this->_slotsPerHour), ($i % $this->_slotsPerHour) * $this->_slotLength, 0,
+                                            $this->days[$j]->month, $this->days[$j]->mday, $this->days[$j]->year);
+                            if ($event->start->timestamp() >= $start && $event->start->timestamp() < $start + 60 * $this->_slotLength || $start == $this->days[$j]->getStamp()) {
                                 $eventCategories[$event->getCategory()] = true;
-                                $row .= '<td class="week-eventbox" style="background-color:' . $categoryColor . ';';
-                                $row .= 'border-color:' . Kronolith::borderColor($categoryColor) . ';" ';
-                                $row .= 'valign="top" ';
-                                $row .= 'width="' . floor(((90 / count($this->days)) / count($this->_currentCalendars)) * ($span / $this->days[$j]->_span[$cid])) . '%" ';
-                                $row .= 'colspan="' . $span . '" rowspan="' . $event->rowspan . '">';
-                                $row .= $event->getLink($this->days[$j]->getStamp());
-                                $row .= '&nbsp;</td>';
+
+                                $row .= '<td class="week-eventbox category' . md5($event->getCategory()) . '" ' .
+                                    'valign="top" ' .
+                                    'width="' . floor(((90 / count($this->days)) / count($this->_currentCalendars)) * ($span / $this->days[$j]->_span[$cid])) . '%" ' .
+                                    'colspan="' . $span . '" rowspan="' . $event->rowspan . '">' .
+                                    $event->getLink($this->days[$j]->getStamp()) . '</td>';
                             }
                         }
                     }
@@ -220,21 +228,17 @@ class Kronolith_WeekView {
                 }
             }
 
-            if ($prefs->getValue('half')) {
-                $rowspan = false;
-                require $template_path . '/day/row.inc';
-            } else {
-                $rowspan = true;
-                if ($i % 2) {
-                    require $template_path . '/day/row_half.inc';
-                } else {
-                    require $template_path . '/day/row.inc';
-                }
-            }
-
-            $first_row = false;
+            $rows[] = array('row' => $row, 'slot' => '<span class="' . $hourclass . '">' . $time . '</span>');
         }
-        require $template_path . '/day/foot.inc';
+
+        require_once 'Horde/Template.php';
+        $template = &new Horde_Template();
+        $template->set('row_height', round(20 / $this->_slotsPerHour));
+        $template->set('rows', $rows);
+        $template->set('show_slots', !$more_timeslots, true);
+        echo $template->fetch($template_path . '/day/rows.html');
+
+        require $template_path . '/category_legend.inc';
     }
 
     /**
@@ -262,7 +266,7 @@ class Kronolith_WeekView {
         }
 
         $this->last = 0;
-        $this->first = 48;
+        $this->first = $this->_slotsPerDay;
         for ($i = $this->startDay; $i <= $this->endDay; $i++) {
             if ($this->days[$i]->last > $this->last) {
                 $this->last = $this->days[$i]->last;
