@@ -2,7 +2,7 @@
 /**
  * Kronolith external API interface.
  *
- * $Horde: kronolith/lib/api.php,v 1.126.2.13 2006/03/06 13:54:04 jan Exp $
+ * $Horde: kronolith/lib/api.php,v 1.126.2.23 2006/10/18 04:22:51 chuck Exp $
  *
  * This file defines Kronolith's external API interface. Other applications
  * can interact with Kronolith through this API.
@@ -12,7 +12,8 @@
 
 $_services['perms'] = array(
     'args' => array(),
-    'type' => '{urn:horde}stringArray');
+    'type' => '{urn:horde}stringArray'
+);
 
 $_services['show'] = array(
     'link' => '%application%/viewevent.php?calendar=|calendar|' .
@@ -21,7 +22,7 @@ $_services['show'] = array(
 
 $_services['browse'] = array(
     'args' => array('path' => 'string'),
-    'type' => '{urn:hash}hashHash',
+    'type' => '{urn:horde}hashHash',
 );
 
 $_services['getFreeBusy'] = array(
@@ -36,7 +37,7 @@ $_services['listCalendars'] = array(
 
 $_services['listEvents'] = array(
     'args' => array('startstamp' => 'int', 'endstamp' => 'int', 'calendar' => 'string', 'showRecurrence' => 'string'),
-    'type' => 'array'
+    'type' => '{urn:horde}stringArray'
 );
 
 $_services['list'] = array(
@@ -242,10 +243,17 @@ function _kronolith_listCalendars($owneronly = false, $permission = null)
     return array_keys(Kronolith::listCalendars($owneronly, $permission));
 }
 
-function _kronolith_list()
+function _kronolith_list($calendar = null)
 {
     require_once dirname(__FILE__) . '/base.php';
-    $ids = Kronolith::listEventIds(0, new Horde_Date(array('year' => 9999, 'month' => 12, 'day' => 31)));
+
+    if (empty($calendar)) {
+        $calendar = Kronolith::getDefaultCalendar();
+    }
+
+    $ids = Kronolith::listEventIds(0,
+                                   new Horde_Date(array('year' => 9999, 'month' => 12, 'day' => 31)),
+                                   $calendar);
     if (is_a($ids, 'PEAR_Error')) {
         return $ids;
     }
@@ -268,7 +276,7 @@ function _kronolith_list()
  *
  * @return array  An array of UIDs matching the action and time criteria.
  */
-function _kronolith_listBy($action, $timestamp, $calendar = '')
+function _kronolith_listBy($action, $timestamp, $calendar = null)
 {
     require_once dirname(__FILE__) . '/base.php';
 
@@ -295,7 +303,7 @@ function _kronolith_listBy($action, $timestamp, $calendar = '')
  *
  * @return integer  The timestamp for this action.
  */
-function _kronolith_getActionTimestamp($uid, $action, $calendar = '')
+function _kronolith_getActionTimestamp($uid, $action, $calendar = null)
 {
     require_once dirname(__FILE__) . '/base.php';
 
@@ -342,77 +350,59 @@ function _kronolith_import($content, $contentType, $calendar = null)
     case 'text/x-icalendar':
     case 'text/x-vcalendar':
     case 'text/x-vevent':
+        require_once 'Horde/iCalendar.php';
+        $iCal = new Horde_iCalendar();
         if (!is_a($content, 'Horde_iCalendar_vevent')) {
-            require_once 'Horde/iCalendar.php';
-            $iCal = &new Horde_iCalendar();
             if (!$iCal->parsevCalendar($content)) {
                 return PEAR::raiseError(_("There was an error importing the iCalendar data."));
             }
-
-            $components = $iCal->getComponents();
-            switch (count($components)) {
-            case 0:
-                return PEAR::raiseError(_("No iCalendar data was found."));
-
-            case 1:
-                $content = $components[0];
-                if (!is_a($content, 'Horde_iCalendar_vevent')) {
-                    return PEAR::raiseError(_("vEvent not found."));
-                }
-                break;
-
-            default:
-                $ids = array();
-                foreach ($components as $content) {
-                    if (is_a($content, 'Horde_iCalendar_vevent')) {
-                        $event = &$kronolith->getEvent();
-                        $event->fromiCalendar($content);
-                        $eventId = $event->save();
-                        if (is_a($eventId, 'PEAR_Error')) {
-                            return $eventId;
-                        }
-                        $ids[] = $eventId;
-                    }
-                }
-                return $ids;
-            }
+        } else {
+            $iCal->addComponent($content);
         }
 
-        $event = &$kronolith->getEvent();
-        $event->fromiCalendar($content);
-        $eventId = $event->save();
-        break;
+        $components = $iCal->getComponents();
+        if (count($components) == 0) {
+            return PEAR::raiseError(_("No iCalendar data was found."));
+        }
 
-    default:
-        return PEAR::raiseError(sprintf(_("Unsupported Content-Type: %s"), $contentType));
+        $ids = array();
+        foreach ($components as $content) {
+            if (is_a($content, 'Horde_iCalendar_vevent')) {
+                $event = &$kronolith->getEvent();
+                $event->fromiCalendar($content);
+                $eventId = $event->save();
+                if (is_a($eventId, 'PEAR_Error')) {
+                    return $eventId;
+                }
+                $ids[] = $event->getUID();
+            }
+        }
+        if (count($ids) == 0) {
+            return PEAR::raiseError(_("No iCalendar data was found."));
+        } else if (count($ids) == 1) {
+            return $ids[0];
+        }
+        return $ids;
     }
 
-    if (is_a($eventId, 'PEAR_Error')) {
-        return $eventId;
-    }
+    return PEAR::raiseError(sprintf(_("Unsupported Content-Type: %s"), $contentType));
 
-    $event = &$kronolith->getEvent($eventId);
-    return $event->getUID();
 }
 
 /**
  * Exports an event, identified by UID, in the requested content type.
  *
  * @param string $uid         Identify the event to export.
- * @param mixed $contentType  What format should the data be in?
- *                            Either a string with one of:
+ * @param string $contentType  What format should the data be in?
+ *                            A string with one of:
  *                            <pre>
- *                             text/calendar
+ *                             text/calendar (VCALENDAR 2.0. Recommended as
+ *                                            this is specified in rfc2445)
+ *                             text/x-vtodo    Seems to be used by horde only.
+ *                                             Do we need this?
+ *                             text/x-vcalendar (old VCALENDAR 1.0 format.
+ *                                              Still in wide use)
  *                             text/x-icalendar
- *                             text/x-vcalendar
- *                            </pre>
- *                            or an array with options:
- *                            <pre>
- *                             'ContentType': as above
- *                             'ENCODING': (optional) character encoding
- *                                         for strings fields.
- *                             'CHARSET':  (optional) charset (ex. UTF-8).
- *                             'VERSION':  (optional) Format version (ex. iCalendar 2.0, vCalendar 1.0).
  *                            </pre>
  *
  * @return string  The requested data.
@@ -433,64 +423,45 @@ function _kronolith_export($uid, $contentType)
     }
 
     $version = '2.0';
-    if (is_array($contentType)) {
-        $options = $contentType;
-        $contentType = $options['ContentType'];
-        unset($options['ContentType']);
-        if (isset($options['VERSION'])) {
-            $version = $options['VERSION'];
-            unset($options['VERSION']);
-        }
-    } else {
-        $options = array();
-    }
-
     switch ($contentType) {
     case 'text/x-vcalendar':
-        // Downgrade version and fall through.
         $version = '1.0';
-
     case 'text/calendar':
     case 'text/x-icalendar':
-        $identity = &$kronolith_shares->getIdentityByShare($kronolith_shares->getShare($event->getCalendar()));
+    case 'text/x-vtodo':
+        $share = &$kronolith_shares->getShare($event->getCalendar());
 
         require_once 'Horde/iCalendar.php';
-        $iCal = &new Horde_iCalendar($version);
+        $iCal = new Horde_iCalendar($version);
+        $iCal->setAttribute('X-WR-CALNAME', $share->get('name'));
 
         // Create a new vEvent.
-        $vEvent = &$event->toiCalendar($iCal, $identity);
-        $vEvent->setParameter('SUMMARY', $options);
-        $vEvent->setParameter('DESCRIPTION', $options);
-        $vEvent->setParameter('LOCATION', $options);
-
+        $vEvent = &$event->toiCalendar($iCal);
         $iCal->addComponent($vEvent);
 
         return $iCal->exportvCalendar();
 
-    default:
-        return PEAR::raiseError(sprintf(_("Unsupported Content-Type: %s"), $contentType));;
     }
+
+    return PEAR::raiseError(sprintf(_("Unsupported Content-Type: %s"), $contentType));;
+
 }
 
 /**
  * Exports a calendar in the requested content type.
  *
  * @param string $calendar    The calendar to export.
- * @param mixed $contentType  What format should the data be in?
- *                            Either a string with one of:
- *                            <pre>
- *                            text/calendar
- *                            text/x-icalendar
- *                            text/x-vcalendar
- *                            </pre>
- *                            or an array with options:
- *                            <pre>
- *                            'ContentType': as above
- *                            'ENCODING': (optional) character encoding for
- *                                        strings fields.
- *                            'CHARSET':  (optional) charset (ex. UTF-8).
- *                            'VERSION':  (optional) Format version (ex. iCalendar 2.0, vCalendar 1.0).
- *                            </pre>
+ * @param string $contentType  What format should the data be in?
+ *                             A string with one of:
+ *                             <pre>
+ *                             text/calendar (VCALENDAR 2.0. Recommended as
+ *                                            this is specified in rfc2445)
+ *                             text/x-vtodo    Seems to be used by horde only.
+ *                                             Do we need this?
+ *                             text/x-vcalendar (old VCALENDAR 1.0 format.
+ *                                              Still in wide use)
+ *                             text/x-icalendar
+ *                             </pre>
  *
  * @return string  The iCalendar representation of the calendar.
  */
@@ -504,48 +475,37 @@ function _kronolith_exportCalendar($calendar, $contentType)
         return PEAR::raiseError(_("Permission Denied"));
     }
 
-    $version = '2.0';
-    if (is_array($contentType)) {
-        $options = $contentType;
-        $contentType = $options['ContentType'];
-        unset($options['ContentType']);
-        if (isset($options['VERSION'])) {
-            $version = $options['VERSION'];
-            unset($options['VERSION']);
-        }
-    } else {
-        $options = array();
+    if ($kronolith->getCalendar() != $calendar) {
+        $kronolith->close();
+        $kronolith->open($calendar);
     }
 
-    $kronolith->open($calendar);
     $events = $kronolith->listEvents(null, null);
 
+    $version = '2.0';
     switch ($contentType) {
     case 'text/x-vcalendar':
-        // Downgrade version and fall through.
         $version = '1.0';
-
     case 'text/calendar':
     case 'text/x-icalendar':
-        require_once 'Horde/iCalendar.php';
+    case 'text/x-vtodo':
+        $share = &$kronolith_shares->getShare($calendar);
 
-        $iCal = &new Horde_iCalendar($version);
-        $identity = &$kronolith_shares->getIdentityByShare($kronolith_shares->getShare($calendar));
+        require_once 'Horde/iCalendar.php';
+        $iCal = new Horde_iCalendar($version);
+        $iCal->setAttribute('X-WR-CALNAME', $share->get('name'));
 
         foreach ($events as $id) {
             $event = &$kronolith->getEvent($id);
-            $vEvent = &$event->toiCalendar($iCal, $identity);
-            $vEvent->setParameter('SUMMARY', $options);
-            $vEvent->setParameter('DESCRIPTION', $options);
-            $vEvent->setParameter('LOCATION', $options);
+            $vEvent = &$event->toiCalendar($iCal);
             $iCal->addComponent($vEvent);
         }
 
         return $iCal->exportvCalendar();
-
-    default:
-        return PEAR::raiseError(sprintf(_("Unsupported Content-Type: %s"), $contentType));
     }
+
+    return PEAR::raiseError(sprintf(_("Unsupported Content-Type: %s"), $contentType));
+
 }
 
 /**
@@ -558,7 +518,7 @@ function _kronolith_exportCalendar($calendar, $contentType)
  */
 function _kronolith_delete($uid)
 {
-    /* Handle an arrray of UIDs for convenience of deleting multiple events at
+    /* Handle an array of UIDs for convenience of deleting multiple events at
      * once. */
     if (is_array($uid)) {
         foreach ($uid as $g) {
@@ -574,13 +534,33 @@ function _kronolith_delete($uid)
     require_once dirname(__FILE__) . '/base.php';
     global $kronolith;
 
-    $event = $kronolith->getByUID($uid);
-    if (is_a($event, 'PEAR_Error')) {
-        return $event;
+    $events = $kronolith->getByUID($uid, true);
+    if (is_a($events, 'PEAR_Error')) {
+        return $events;
     }
 
-    if (!array_key_exists($event->getCalendar(),
-                          Kronolith::listCalendars(false, PERMS_DELETE))) {
+    /* First try the user's own calendars. */
+    $ownerCalendars = Kronolith::listCalendars(true, PERMS_DELETE);
+    $event = null;
+    foreach ($events as $ev) {
+    	if (isset($ownerCalendars[$ev->getCalendar()])) {
+            $event = $ev;
+            break;
+    	}
+    }
+
+    /* If not successful, try all calendars the user has access too. */
+    if (empty($event)) {
+    	$deletableCalendars = Kronolith::listCalendars(false, PERMS_DELETE);
+    	foreach ($events as $ev) {
+            if (isset($deletableCalendars[$ev->getCalendar()])) {
+                $event = $ev;
+                break;
+            }
+    	}
+    }
+
+    if (empty($event)) {
         return PEAR::raiseError(_("Permission Denied"));
     }
 
@@ -623,7 +603,7 @@ function _kronolith_replace($uid, $content, $contentType)
     case 'text/x-vevent':
         if (!is_a($content, 'Horde_iCalendar_vevent')) {
             require_once 'Horde/iCalendar.php';
-            $iCal = &new Horde_iCalendar();
+            $iCal = new Horde_iCalendar();
             if (!$iCal->parsevCalendar($content)) {
                 return PEAR::raiseError(_("There was an error importing the iCalendar data."));
             }
@@ -728,13 +708,33 @@ function _kronolith_updateAttendee($response)
         return $uid;
     }
 
-    $event = $kronolith->getByUID($uid);
-    if (is_a($event, 'PEAR_Error')) {
-        return $event;
+    $events = $kronolith->getByUID($uid, true);
+    if (is_a($events, 'PEAR_Error')) {
+        return $events;
     }
 
-    if (!array_key_exists($event->getCalendar(),
-                          Kronolith::listCalendars(false, PERMS_EDIT))) {
+    /* First try the user's own calendars. */
+    $ownerCalendars = Kronolith::listCalendars(true, PERMS_EDIT);
+    $event = null;
+    foreach ($events as $ev) {
+    	if (isset($ownerCalendars[$ev->getCalendar()])) {
+            $event = $ev;
+            break;
+    	}
+    }
+
+    /* If not successful, try all calendars the user has access too. */
+    if (empty($event)) {
+    	$editableCalendars = Kronolith::listCalendars(false, PERMS_EDIT);
+    	foreach ($events as $ev) {
+            if (isset($editableCalendars[$ev->getCalendar()])) {
+                $event = $ev;
+                break;
+            }
+    	}
+    }
+
+    if (empty($event)) {
         return PEAR::raiseError(_("Permission Denied"));
     }
 
@@ -745,7 +745,7 @@ function _kronolith_updateAttendee($response)
         $atnames = array($atnames);
     }
     foreach ($atnames as $index => $attendee) {
-        $attendee = str_replace('MAILTO:', '', $attendee);
+        $attendee = str_replace('mailto:', '', String::lower($attendee));
         if ($event->hasAttendee($attendee)) {
             $event->addAttendee($attendee, KRONOLITH_PART_IGNORE, Kronolith::responseFromICal($atparms[$index]['PARTSTAT']));
         }
